#include <active_check.h>
#include <board_def.h>
#include <eth_init.h>

#define GPIO_ETH_POWER  xcat(GPIO, ETH_POWER_GPIO_N)
#define RCC_ETH_POWER   xcat(RCC_APB2Periph_GPIO, ETH_POWER_GPIO_N)
#define PIN_ETH_POWER   xcat(GPIO_Pin_, ETH_POWER_PIN_N)
#define ETH_POWER_OFF   (!(ETH_POWER_ON))

static int ETHinterfaceMIIconfigure(void);
static int ETHinterfaceRemapedMIIconfigure(void);
static int ETHinterfaceRMIIconfigure(void);
static int ETHinterfaceRemapedRMIIconfigure(void);
static void ETHpowerConfigure(void);

static const int maxAttempts = 1000000;

int ETHconfigureInterface() {
  int result;

  if (ETH_PHY_INTERFACE == ETH_MII)
    result = ETHinterfaceMIIconfigure();
  else if (ETH_PHY_INTERFACE == ETH_MII_REMAPED)
    result = ETHinterfaceRemapedMIIconfigure();
  else if (ETH_PHY_INTERFACE == ETH_RMII)
    result = ETHinterfaceRMIIconfigure();
  else if (ETH_PHY_INTERFACE == ETH_RMII_REMAPED)
    result = ETHinterfaceRemapedRMIIconfigure();
  else
    result = -1;
  if (result < 0)
    return -1;
  ETHpowerConfigure();
  ETHpowerUp();
  ETH_DeInit();
  ETH_SoftwareReset();
  active_check(ETH_GetSoftwareResetStatus() == RESET, maxAttempts);
  return 0;
}

int ETHinterfaceMIIconfigure() {
  /* Uzupełnij implementację. */
  return -1;
}

int ETHinterfaceRemapedMIIconfigure() {
  GPIO_InitTypeDef GPIO_InitStructure;

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA |
                         RCC_APB2Periph_GPIOB |
                         RCC_APB2Periph_GPIOC |
                         RCC_APB2Periph_GPIOD |
                         /* RCC_APB2Periph_GPIOE | */
                         RCC_APB2Periph_AFIO,
                         ENABLE);

  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_ETH_MAC |
                        RCC_AHBPeriph_ETH_MAC_Tx |
                        RCC_AHBPeriph_ETH_MAC_Rx,
                        ENABLE);

  if (ETH_PHY_CLOCK == ETH_CLOCK_AUTONOMOUS) {
  }
  else if (ETH_PHY_CLOCK == ETH_CLOCK_PROVIDED_BY_MICRO) {
    if (HSE_VALUE == 25000000) {
      /* Jeśli mamy kwarc 25 MHz, można ominąć PLL3. */
      RCC_MCOConfig(RCC_MCO_XT1);
    }
    else {
      /* PLL3: PLL3CLK = (HSE / 5) * 10 = 50 MHz */
      RCC_PLL3Config(RCC_PLL3Mul_10);
      RCC_PLL3Cmd(ENABLE);
      active_check(RCC_GetFlagStatus(RCC_FLAG_PLL3RDY), maxAttempts);
      /* MCO (PA8) = PLL3CLK / 2 = 25 MHz */
      RCC_MCOConfig(RCC_MCO_PLL3CLK_Div2);
    }
  }
  else {
    return -1;
  }

  GPIO_PinRemapConfig(GPIO_Remap_ETH, ENABLE);
  GPIO_ETH_MediaInterfaceConfig(GPIO_ETH_MediaInterface_MII);

  /* Linie wejściowe
     ETH_MII_CRS    - PA0 - Odbieranie carrier sense jest wymagane
                            w half-duplex.
     ETH_MII_RX_CLK - PA1
     ETH_MII_COL    - PA3
     ETH_MII_RX_ER  - PB10
     ETH_MII_TX_CLK - PC3
     ETH_MII_RX_DV  - PD8
     ETH_MII_RXD0   - PD9
     ETH_MII_RXD1   - PD10
     ETH_MII_RXD2   - PD11
     ETH_MII_RXD3   - PD12 */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_InitStructure.GPIO_Speed = 0;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_3;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9 |
                                GPIO_Pin_10 | GPIO_Pin_11 |
                                GPIO_Pin_12;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  /* Linie wyjściowe
     ETH_MII_MDIO    - PA2
     ETH_MII_PPS_OUT - PB5 - niepodłączone do PHY
     ETH_MII_TXD3    - PB8
     ETH_MII_TX_EN   - PB11
     ETH_MII_TXD0    - PB12
     ETH_MII_TXD1    - PB13
     ETH_MII_MDC     - PC1
     ETH_MII_TXD2    - PC2
     MCO             - PA8 - zegar jako ostatni */
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = /* GPIO_Pin_5 | */
                                GPIO_Pin_8 | GPIO_Pin_11 |
                                GPIO_Pin_12 | GPIO_Pin_13;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  return 0;
}

int ETHinterfaceRMIIconfigure(void) {
  /* MCO (PA8) = PLL3CLK = 50 MHz
  RCC_MCOConfig(RCC_MCO_PLL3CLK); */
  /* Uzupełnij implementację. */
  return -1;
}

int ETHinterfaceRemapedRMIIconfigure(void) {
  /* Uzupełnij implementację. */
  return -1;
}

/* Konfiguruj wyprowadzenie PWRDWN modułu ethernetowego (PHY). */
void ETHpowerConfigure() {
  GPIO_InitTypeDef GPIO_InitStruct;

  RCC_APB2PeriphClockCmd(RCC_ETH_POWER, ENABLE);
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_Out_OD;
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStruct.GPIO_Pin = PIN_ETH_POWER;
  GPIO_Init(GPIO_ETH_POWER, &GPIO_InitStruct);
}

/* Wyłącz zasilanie modułu ethernetowego. */
void ETHpowerDown() {
  /* ETH_PowerDownCmd(ENABLE); */
  GPIO_WriteBit(GPIO_ETH_POWER, PIN_ETH_POWER, ETH_POWER_ON);
}

/* Włącz zasilanie modułu ethernetowego. Po resecie wyjście jest
   w stanie niskim, ale na wszelki wypadek... */
void ETHpowerUp() {
  GPIO_WriteBit(GPIO_ETH_POWER, PIN_ETH_POWER, ETH_POWER_ON);
}
