#include <board_conf.h>
#include <board_init.h>
#include <delay.h>
#include <error.h>
#include <eth_init.h>
#include <lcd_util.h>
#include <lcd_util_ex.h>
#include <led.h>
#include <lwip_util.h>
#include <time_util.h>
#include <stdio.h>

int main() {
  static struct netif netif;
  static struct ethnetif ethnetif = {ETH_PHY_ADDRESS};
  uint8_t confBit;

  Delay(1000000);       /* Czekaj na JTAG ok. 0,5 s. */
  confBit = GetConfBit();
  AllPinsDisable();
  LEDconfigure();
  RedLEDon();
  LCDconfigure();
  IRQprotectionConfigure();

  LCDwrite("Clock ");
  ErrorResetable(ClockConfigure(MAIN_CLOCK_MHZ), 1);
  LCDwrite("PASS\n");
  LCDwrite("Local Time ");
  ErrorPermanent(LocalTimeConfigure(), 2);
  LCDwrite("PASS\n");
  LCDwrite("Ethernet ");
  ErrorResetable(ETHconfigureInterface(), 4);
  LCDwrite("PASS\n");

  /* Każde urządzenie powinno mieć inny adres MAC, który jest
     zapisywany w porządku grubokońcówkowym. */
  LCDwrite("lwIP stack ");
  netif.hwaddr[0] = (ETH_MAC >> 40) & 0xff;
  netif.hwaddr[1] = (ETH_MAC >> 32) & 0xff;
  netif.hwaddr[2] = (ETH_MAC >> 24) & 0xff;
  netif.hwaddr[3] = (ETH_MAC >> 16) & 0xff;
  netif.hwaddr[4] = (ETH_MAC >> 8) & 0xff;
  netif.hwaddr[5] = (ETH_MAC & 0xff) + confBit;
  if (!confBit) {
    IP4_ADDR(&netif.ip_addr, 192, 168,  51,  84);
    IP4_ADDR(&netif.netmask, 255, 255, 255, 240);
    IP4_ADDR(&netif.gw,      192, 168,  51,  81);
  }
  else {
    /* Użyj DHCP. */
    IP4_ADDR(&netif.ip_addr, 0, 0, 0, 0);
    IP4_ADDR(&netif.netmask, 0, 0, 0, 0);
    IP4_ADDR(&netif.gw,      0, 0, 0, 0);
  }
  ErrorResetable(LWIPinterfaceInit(&netif, &ethnetif), 5);
  LWIPtimersStart();
  LCDwrite("PASS\n");

  /* Na uzyskanie adresu IP czekaj maksymalnie
     10 sekund lub 4 próby. */
  LCDwrite("IP ");
  ErrorResetable(DHCPwait(&netif, 10, 4), 6);
  LCDwriteIP(&netif.ip_addr);
  RedLEDoff();

  for (;;) {
    LCDgoto(6, 0);
    LCDwriteRXTX(ethnetif.RX_packets, ethnetif.RX_errors,
                 ethnetif.TX_packets, ethnetif.TX_errors);
    Delay(1000);
  }
}
