#include <board_conf.h>
#include <board_init.h>
#include <delay.h>
#include <error.h>
#include <eth_init.h>
#include <lcd_util.h>
#include <lcd_util_ex.h>
#include <led.h>
#include <lwip_util.h>
#include <rtc.h>
#include <sntp_client.h>
#include <time_util.h>

/* Aby przetestować aplikację, na liście jest serwer
   nieistniejący oraz serwer nieobsługujący NTP. */
#define COUNT  5
static const char * const serverTbl[COUNT] = {
  "wikipedia.org",
  "nie.ma.takiego.serwera",
  "wikipedia.org",
  "tempus1.gum.gov.pl",
  "tempus2.gum.gov.pl"
};
static int status[COUNT] = {
  SNTP_NOT_RUNNING,
  SNTP_NOT_RUNNING,
  SNTP_NOT_RUNNING,
  SNTP_NOT_RUNNING,
  SNTP_NOT_RUNNING
};

/* Wykonuje się w przerwaniu RTC. */
static void ClockCallback(void) {
  static int f = 0;
  int i;
  struct tm tm;

  LCDgoto(0, 0);
  if (GetRealTimeClock(&tm) < 0)
    LCDwrite("RTC error             ");
  else
    LCDwriteTime(&tm);
  for (i = 0; i < COUNT; ++i) {
    LCDgoto(2 + i, 0);
    if (f == 0) {
      LCDwrite(serverTbl[i]);
      LCDwrite("                     ");
    }
    else if (f == 2 || f == 3) {
      switch (status[i]) {
        case SNTP_NOT_RUNNING:
          LCDwrite("SNTP not running      ");
          break;
        case SNTP_SYNCHRONIZED:
          LCDwrite("Time synchronized     ");
          break;
        case SNTP_IN_PROGRESS:
          LCDwrite("SNTP in progress      ");
          break;
        case SNTP_DNS_ERROR:
          LCDwrite("DNS error             ");
          break;
        case SNTP_MEMORY_ERROR:
          LCDwrite("Memory error          ");
          break;
        case SNTP_UDP_ERROR:
          LCDwrite("UDP error             ");
          break;
        case SNTP_NO_RESPONSE:
          LCDwrite("SNTP no response      ");
          break;
        case SNTP_RESPONSE_ERROR:
          LCDwrite("SNTP response error   ");
          break;
        case SNTP_RTC_ERROR:
          LCDwrite("RTC error             ");
          break;
      }
    }
  }
  f = (f + 1) & 3;
}

int main() {
  static struct netif netif;
  static struct ethnetif ethnetif = {ETH_PHY_ADDRESS};
  int i;
  uint8_t confBit;

  Delay(1000000);       /* Czekaj na JTAG ok. 0,5 s. */
  confBit = GetConfBit();
  AllPinsDisable();
  LEDconfigure();
  RedLEDon();
  LCDconfigure();
  IRQprotectionConfigure();

  ErrorResetable(ClockConfigure(MAIN_CLOCK_MHZ), 1);
  ErrorPermanent(LocalTimeConfigure(), 2);
  ErrorResetable(RTCconfigure(), 3);
  RTCsetCallback(ClockCallback);
  ErrorResetable(ETHconfigureInterface(), 4);

  /* Każde urządzenie powinno mieć inny adres MAC. */
  netif.hwaddr[0] = (ETH_MAC >> 40) & 0xff;
  netif.hwaddr[1] = (ETH_MAC >> 32) & 0xff;
  netif.hwaddr[2] = (ETH_MAC >> 24) & 0xff;
  netif.hwaddr[3] = (ETH_MAC >> 16) & 0xff;
  netif.hwaddr[4] = (ETH_MAC >> 8) & 0xff;
  netif.hwaddr[5] = (ETH_MAC & 0xff) + confBit;
  if (!confBit) {
    IP4_ADDR(&netif.ip_addr, 192, 168,  51,  84);
    IP4_ADDR(&netif.netmask, 255, 255, 255, 240);
    IP4_ADDR(&netif.gw,      192, 168,  51,  81);
  }
  else {
    /* Użyj DHCP. */
    IP4_ADDR(&netif.ip_addr, 0, 0, 0, 0);
    IP4_ADDR(&netif.netmask, 0, 0, 0, 0);
    IP4_ADDR(&netif.gw,      0, 0, 0, 0);
  }
  ErrorResetable(LWIPinterfaceInit(&netif, &ethnetif), 5);
  LWIPtimersStart();
  ErrorResetable(DHCPwait(&netif, 10, 4), 6);
  RedLEDoff();

  for (i = 0; i < COUNT; ++i) {
    Delay(10000000);
    SNTPclientStart(serverTbl[i], &status[i]);
  }
  for (;;);
}
